angular.module( 'rcp-map', ['ionic', 'rcp-api-adapter'])

.config(function($stateProvider, $urlRouterProvider) {
    $stateProvider
    .state('tabs.map', {
        url: '/map',
        views: {
            'map-tab': {
                controller: 'MapCtrl',
                templateUrl: 'app/components/map/map.html',
            }
        }
    });
})

.controller('MapCtrl', function(api, $scope, $ionicPopup, $state, $compile, mapMarker, $ionicLoading, $ionicModal, loadingIndicator) {
    $scope.$on('$ionicView.afterEnter', function(){
		var mapOptions = {
            zoom: 14,
            mapTypeId: google.maps.MapTypeId.ROADMAP
        };

        var map = new google.maps.Map(document.getElementById("map"), mapOptions);
        $scope.map = map;

        initialize();
    });

    $scope.locationData = [];

	var mapPins = new Array();
	var mapInfoWindows = new Array();
	var mapPinListeners = new Array();
	var gpsPosition;

    function initialize() {
		// this is removed after marking all pins
		loadingIndicator.showLoadingScreen($scope, 'Finding nearest car parks...');

        var options = {maximumAge: 60000, frequency: 5000, timeout: 10000, enableHighAccuracy:true};
        navigator.geolocation.getCurrentPosition(function(pos) {
            gpsPosition = new google.maps.LatLng(pos.coords.latitude, pos.coords.longitude);

            var myLocation = mapMarker.addMarker(-1, "<strong>You are here</strong>", gpsPosition, $scope.map, true);
			mapPins.push(myLocation.marker);

            getNearest(pos);
        }, function(error) {
            $ionicLoading.hide();

            if (error.code == 3) {
                $scope.showAlert('Error', 'We couldn\'t find a satellite signal. Do you have GPS enabled and a clear view of the sky?');
            } else {
                $scope.showAlert('Error', 'Unable to get location: ' + error.message);
            }
        }, options);
    }

	$scope.showLocation = function() {
		removeMarkers();
		initialize();
	}

    function getNearest(pos) {
        var payload = {
            latitude: pos.coords.latitude,
            longitude: pos.coords.longitude 
        };

        var locations = api.getNearestLocations(payload);
        locations.then(function(response) {
            $scope.locationData = response.data;
            markLocations(response.data);
        }, function(reason) {
            $ionicLoading.hide();
            $scope.showAlert(reason.error);
        });
    }

    function markLocations(data) {
        var nearestLocs = Object.keys(data).map(function(k) { return data[k] }); 
        var nearestLocsG = new Array(); 

        var id = 0;
        nearestLocs.forEach(function(location) {
            var position = new google.maps.LatLng(location.car_parks_lat, location.car_parks_lng);
            var content = "<img style=\"margin-right: 8px; margin-bottom: 8px; vertical-align: middle;\" src=\"/app/client/assets/img/rcp-icon-small.png\"><strong>"+location.car_parks_car_park+"</strong><br>Car park id: <strong>"+location.front_carpark_id+"</strong>";

            var pin = mapMarker.addMarker(id, content, position, $scope.map, false, function(id) {
                $scope.car_park_title = $scope.locationData[id].car_parks_car_park;
                $scope.address_line1 = $scope.locationData[id].car_parks_address1;
                $scope.address_line2 = $scope.locationData[id].car_parks_address2;
                $scope.address_line3 = $scope.locationData[id].car_parks_address3;
                $scope.city = $scope.locationData[id].car_parks_city;
                $scope.postcode = $scope.locationData[id].car_parks_postcode;
                $scope.times = $scope.locationData[id].car_parks_opening_times;
                $scope.price = $scope.locationData[id].price;
                $scope.location_code = $scope.locationData[id].front_carpark_id;

                $scope.openModal();
            });

			mapPins.push(pin.marker);
			mapInfoWindows.push(pin.infoWindow);
            mapPinListeners.push(pin.listener);
            nearestLocsG.push(position);
            id++;
        });
 
		$ionicLoading.hide();
		autoCenter(nearestLocsG);
    }

	function removeMarkers() {
		for (var i = 0; i < mapPins.length; i++) {
			mapPins[i].setMap(null);
			google.maps.event.removeListener(mapPinListeners[i]);
		}

		mapPins = Array();
		mapPinListeners = Array();

		delete mapInfoWindows;
		mapInfoWindows = Array();
	}

	$scope.showAlert = function(title, message, callback) {
		var alertPopup = $ionicPopup.alert({
			title: title,
			template: message 
		});

		alertPopup.then(function(res) {
			if (callback) {
				callback(res);
			}
		});
	};

	function autoCenter(data) {
		var latlngbounds = new google.maps.LatLngBounds();
		for (var i = 0; i < data.length; i++) {
			latlngbounds.extend(data[i]);
		}

		$scope.map.setCenter(latlngbounds.getCenter(), $scope.map.fitBounds(latlngbounds));

		if (data.length < 3) {
			$scope.map.setZoom(16);
		}
	}

	$ionicModal.fromTemplateUrl('templates/map-marker.html', {
		scope: $scope,
		animation: 'slide-in-up'
	}).then(function(modal) {
		$scope.modal = modal;
	});

	$scope.openModal = function(){
		$scope.modal.show();
	}

	$scope.closeModal = function(){
		$scope.modal.hide();
	}

	/*
	 * Search Functionality
	 */

	$scope.showSearchPopup = function() {
		displaySearch(search);
	}

	function displaySearch(callback) {
		$scope.data = {}
		$contentTemplate = '<input type="text" focus ng-model="data.search" placeholder="City or Postcode" class="map-search-input">';

		var search = $ionicPopup.show({
			template: $contentTemplate,
			title: 'Car Park Search',
			subTitle: 'Search by <strong>city</strong> or <strong>postcode</strong>',
			scope: $scope,
			buttons: [
				{ text: 'Cancel' },
				{
					text: '<b>Search</b>',
					type: 'button-positive',
					onTap: function(e) {
						if (!$scope.data.search) {
							e.preventDefault();
						} else {
							return $scope.data.search;
						}
					}
				}
			]
		});

		search.then(function(res) {
			if (res) {
				callback(res);
			}
		});
	};

	function search($query) {
		loadingIndicator.showLoadingScreen($scope, 'Searching for car parks...');

		var searchLocations = api.getSearchLocations({ query: $query });
        searchLocations.then(function(response) {
            $scope.modal.hide();

			if (Array.isArray(response.data)) {
				$ionicLoading.show({ template: 'Found '+response.data.length+' car parks', noBackdrop: true, duration: 2000 });	
				removeMarkers();

				$scope.locationData = response.data;
				markLocations(response.data);
			} else {
				$ionicLoading.show({ template: response.data, noBackdrop: true, duration: 2000 });	
			}
        }, function(reason) {
            $ionicLoading.hide();
            if (reason.status == 400) {
                errors = JSON.stringify(reason.errors);
                $scope.showAlert('Some required fields are missing: ' + errors); 
            } else {
                $scope.showAlert(reason.errors); 
            }
        });
	}
});
